@@ -1,16 +1,25 @@
 /***********************************************************************
  *   Code is based on wmppp, wmload, wmtime, wmcp, and asbutton
  *   Author: Edward H. Flora <ehflora@ksu.edu>
- *   Ver 0 Rel 3    March 6, 1999
+ *   Ver 0 Rel 6.1    Jan 23, 2005
  *
  *   Contributors:
+ *              Christian 'Greek0' Aichinger <Greek0@gmx.net>
+ *                  Did some code cleanup and fixed several memory leaks.
+ *              Ralf Horstmann <ralf.horstmann@gmx.de>
+ *                  Added ability to load pixmaps at startup,
+ *                  without having to re-compile
+ *              Michael Cohrs <camico@users.sourceforge.net> 
+ *                  Added Tool Tips, and updated graphics
+ *              Bruno Essmann <essmann@users.sourceforge.net>)
+ *                  Creator of wmpager
+ *              Casey Harkins <charkins@cs.wisc.edu> 
+ *                  Bug fix reading config file path - 3/6/99
+ *                  Added button-presses, and other - denoted by *charkins*
  *              Ben Cohen <buddog@aztec.asu.edu>
  *                  original author of wmcp (et al.)
  *              Thomas Nemeth <tnemeth@multimania.com>
  *                  contributor to wmcp
- *              Casey Harkins <charkins@cs.wisc.edu> 
- *                  Bug fix reading config file path - 3/6/99
- *                  Added button-presses, and other - denoted by *charkins*
  *              Michael Henderson <mghenderson@lanl.gov>
  *                  Application ideas, suggestions
  *              Ryan ?? <pancake@mindspring.com> 
@@ -24,30 +33,25 @@
  *    development of wmbutton.  I'm listing them here partially as thanks for 
  *    helping out, catching bugs in the code, etc.
  ***********************************************************************/
-#include <Xlib.h>
-#include <Xutil.h>
-#include <xpm.h>
-#include <extensions/shape.h>
-#include <keysym.h>
 #include <stdio.h>
 #include <math.h>
 #include <stdlib.h>
 #include <string.h>
 #include <ctype.h>
+#include <unistd.h>
 
-#include "backdrop.xpm"           // background graphic
-#include "buttons.xpm"            // graphic of 9 buttons
-#include "mask.xbm"
+#include "wmbutton.h"
 
-#define CONFIGFILEMAX 128
-#define CONFFILENAME  "/.wmbutton"
-#define VER           0
-#define REL           4
+#include "backdrop.xpm"           /* background graphic */
+#include "buttons.xpm"            /* graphic of 9 buttons */
+#include "mask.xbm"               /* Border Graphic */
 
-// #define MIDMOUSE
-#define LMASK         0
-#define MMASK         10
-#define RMASK         20
+/*************** Function Prototypes ***********************************/
+void redraw(void);                 
+void getPixmaps(void);
+int  whichButton(int x, int y);   // determine which button has been pressed
+void SetWmHints();
+void SetClassHints();
 
 /***********************************************************************
  * 		Globals..    OK.. there's too many globals.
@@ -60,10 +64,7 @@ GC gc;
 int depth;
 Pixel bg_pixel, fg_pixel;
 
-XSizeHints xsizehints;
-XWMHints * xwmhints;
-XClassHint xclasshint;
-char configfile[CONFIGFILEMAX]; 
+struct Config_t Config;
 
 typedef struct _XpmIcon {
   Pixmap pixmap;
@@ -71,9 +72,6 @@ typedef struct _XpmIcon {
   XpmAttributes attributes;
 } XpmIcon;
 
-
-Pixmap pixmask;
-
 typedef struct _button_region {
   int x,y;
   int i,j;
@@ -83,81 +81,40 @@ ButtonArea button_region[9];
 
 XpmIcon template, visible, buttons;
 
-int numb_of_apps = 9;
-int N = 1;		/* Button number pressed to goto app #     */
 int border  = 0;
-int Verbose = 0;
-int mmouse  = 0;
 int button_pressed = -1;	/* button to be drawn pressed *charkins*/
 
 char *app_name = "wmbutton";
 
-/*************** Function Prototypes ***********************************/
-void RunAppN(int app);             // function to run app N as found in conf file
-char *Parse(int app);              // parse data in config file
-void redraw(void);                 
-void getPixmaps(void);
-int  whichButton(int x, int y);    // determine which button has been pressed
-int  flush_expose(Window w);
-void show_usage(void);             // show usage message to stderr
-char *readln(FILE *fp);            // read line from file, return pointer to it
-
 /***********************************************************************
  * 		Main
  ***********************************************************************/
 int main( int argc, char ** argv ) {
+
   XEvent report;
   XGCValues xgcValues;	
   XTextProperty app_name_atom;
+  XSizeHints xsizehints;
+
+  Pixmap pixmask;
 
   int dummy = 0;
-  int i;
-  char Geometry_str[64] = "64x64+0+0";
-  char Display_str[64] = "";
+  int N = 1;		        /* Button number pressed to goto app # */
   
-  strcpy(configfile, getenv("HOME"));  // Added by Casey Harkin, 3/6/99
-  strcat(configfile, CONFFILENAME);    // Fixed Bug - didn't look in home directory
-                                       // but startup directory
+  /* Added for Tool Tip Support */
+  long nTooltipShowDelay = TOOLTIP_SHOW_DELAY;
+  long nTooltipReshowDelay = TOOLTIP_RESHOW_DELAY;
+  long nTooltipTimer = -1;
+  long nTooltipHideTimer = -1;
+  long nNow;
+  int nTooltipButton = 0, nTooltipX = 0, nTooltipY = 0;
 
   /* Parse Command Line Arguments */  
-  for ( i=1; i<argc; i++ ) {
-    if ( *argv[i] == '-' ) {
-      switch ( *(argv[i]+1) ) {
-      case 'v':                        // Turn on Verbose (debugging) Mode
-	Verbose = 1;
-	break;
-      case 'g':                        // Set Geometry Options
-	if ( ++i >= argc ) show_usage();
-	sscanf(argv[i], "%s", Geometry_str);
-	if ( Verbose ) printf("Geometry is: %s\n", Geometry_str);
-	break;
-      case 'd':                        // Set display 
-	if ( ++i >= argc ) show_usage();
-	sscanf(argv[i], "%s", Display_str);
-	if ( Verbose ) printf("Display is: %s\n", Display_str);
-	break;
-      case 'h':                        // Show Help Message
-	show_usage();
-	break;
-      case 'f':                        // use config file <filename>
-	if ( ++i >= argc ) show_usage();
-	sscanf(argv[i], "%s", configfile);
-	if ( Verbose ) printf("Using Config File: %s\n", configfile);
-	break;
-      case 'm':                        // Enable Middle Mouse
-	mmouse = 1;
-	break;
-      default:                         // other, unknown, parameters
-	show_usage();
-	break;
-      }
-    }
-  }
-    
-  if ( (display = XOpenDisplay(Display_str)) == NULL ) {
-    fprintf(stderr,"Fail: XOpenDisplay for %s\n", Display_str);	
-    exit(-1);
-  }
+  parseargs(argc, argv);
+
+  /* Open Display */
+  if ( (display = XOpenDisplay(Config.Display_str)) == NULL ) 
+    err_mess(FAILDISP, Config.Display_str);
   
   screen  = DefaultScreen(display);
   rootwin = RootWindow(display,screen);
@@ -172,7 +129,7 @@ int main( int argc, char ** argv ) {
   
   /* Parse Geometry string and fill in sizehints fields */
   XWMGeometry(display, screen, 
-	      Geometry_str, 
+	      Config.Geometry_str, 
 	      NULL, 	
 	      border, 
 	      &xsizehints,
@@ -189,10 +146,9 @@ int main( int argc, char ** argv ) {
 				  xsizehints.width,
 				  xsizehints.height,
 				  border,
-				  fg_pixel, bg_pixel) ) == 0 ) {
-    fprintf(stderr,"Fail: XCreateSimpleWindow\n");	
-    exit(-1);
-  }
+				  fg_pixel, bg_pixel) ) == 0 )
+    err_mess(FAILSWIN, NULL);
+
   
   if ( (iconwin = XCreateSimpleWindow(display,
 				      win,
@@ -201,10 +157,8 @@ int main( int argc, char ** argv ) {
 				      xsizehints.width,
 				      xsizehints.height,
 				      border,
-				      fg_pixel, bg_pixel) ) == 0 ) {
-    fprintf(stderr,"Fail: XCreateSimpleWindow\n");	
-    exit(-1);
-  }
+				      fg_pixel, bg_pixel) ) == 0 )
+    err_mess(FAILICON, NULL);
 
   /* Set up shaped windows */
   /*Gives the appicon a border so you can grab and move it. */
@@ -213,9 +167,9 @@ int main( int argc, char ** argv ) {
 					 win,
 					 mask_bits,
 					 mask_width,
-					 mask_height) )  == 0 ) {
-    fprintf(stderr,"Fail: XCreateBitmapFromData\n");
-  }
+					 mask_height) )  == 0 )
+    err_mess(FAILXPM,NULL);
+
   
   XShapeCombineMask(display, win, ShapeBounding, 0, 0, pixmask, ShapeSet );
   XShapeCombineMask(display, iconwin, ShapeBounding, 0, 0, pixmask, ShapeSet);
@@ -225,47 +179,38 @@ int main( int argc, char ** argv ) {
 
   /* Interclient Communication stuff */
   /* Appicons don't work with out this stuff */
-  xwmhints = XAllocWMHints();
-  xwmhints->flags = WindowGroupHint | IconWindowHint | StateHint;	
-  xwmhints->icon_window = iconwin;
-  xwmhints->window_group = win;
-  xwmhints->initial_state = WithdrawnState;  
-  XSetWMHints( display, win, xwmhints );
+  SetWmHints();
+  SetClassHints();
 
-  xclasshint.res_name = "wmbutton";
-  xclasshint.res_class = "Wmbutton";
-  XSetClassHint( display, win, &xclasshint );
 
   XSetWMNormalHints( display, win, &xsizehints );
   
   /* Tell window manager what the title bar name is. We never see */
   /* this anyways in the WithdrawnState      */
-  if ( XStringListToTextProperty(&app_name, 1, &app_name_atom) == 0 ) {
-    fprintf(stderr,"%s: Can't set up window name\n", app_name);
-    exit(-1);
-  }
+  if ( XStringListToTextProperty(&app_name, 1, &app_name_atom) == 0 )
+    err_mess(FAILWNAM,app_name);
   XSetWMName( display, win, &app_name_atom );
   
   /* Create Graphic Context */	
   if (( gc = XCreateGC(display, win,(GCForeground | GCBackground), &xgcValues))
-       == NULL ) {
-    fprintf(stderr,"Fail: XCreateGC\n");	
-    exit(-1);
-  }
+       == NULL )
+    err_mess(FAILGC,NULL);
 
   /* XEvent Masks. We want both window to process X events */
   XSelectInput(display, win,
 	       ExposureMask | 
 	       ButtonPressMask | 
-		   ButtonReleaseMask |		/* added ButtonReleaseMask *charkins*/
+	       ButtonReleaseMask |    	/* added ButtonReleaseMask *charkins*/
 	       PointerMotionMask |
-	       StructureNotifyMask );  
+	       StructureNotifyMask |
+	       LeaveWindowMask );  
   XSelectInput(display, iconwin,
 	       ExposureMask | 
 	       ButtonPressMask | 
-		   ButtonReleaseMask |		/* added ButtonReleaseMask *charkins*/
+	       ButtonReleaseMask |	/* added ButtonReleaseMask *charkins*/
 	       PointerMotionMask |
-	       StructureNotifyMask );
+	       StructureNotifyMask |
+	       LeaveWindowMask );
   
   /* Store the 'state' of the application for restarting */
   XSetCommand( display, win, argv, argc );	
@@ -274,106 +219,171 @@ int main( int argc, char ** argv ) {
   /* ConfigureNotify */
   XMapWindow( display, win );
 
+  /* Initialize Tooltip Support */
+  initTime();
+  initTooltip(!Config.bTooltipDisable, Config.szTooltipFont, Config.bTooltipSwapColors);
+
   /* X Event Loop */
   while (1) {
-    XNextEvent(display, &report );
-    switch (report.type) {
-    case Expose:
-      if (report.xexpose.count != 0) {	
-	break;
-      }
-      if ( Verbose ) fprintf(stdout,"Event: Expose\n");	
-      redraw();
-      break;      
-    case ConfigureNotify:
-      if ( Verbose ) fprintf(stdout,"Event: ConfigureNotify\n");	
-      redraw();
-      break;
-    case ButtonPress:	/* draw button pressed, don't launch *charkins*/
-      switch (report.xbutton.button) {
-      case Button1:
-	N = whichButton(report.xbutton.x, report.xbutton.y );
-	if ( (N >= 0) && (N <= numb_of_apps) ) {
-	  button_pressed = N + LMASK;
-	  redraw();
+    while (XPending(display) || nTooltipTimer == -1) {
+      XNextEvent(display, &report );
+      switch (report.type) {
+      case Expose:
+	if (report.xexpose.count != 0) {	
+	  break;
 	}
-	if ( Verbose ) 
-	  fprintf(stdout,"Button 1:x=%d y=%d N=%d\n", 
-		  report.xbutton.x, report.xbutton.y, N+LMASK);	
+	if ( Config.Verbose ) fprintf(stdout,"Event: Expose\n");	
+	redraw();
+	break;      
+
+      case ConfigureNotify:
+	if ( Config.Verbose ) fprintf(stdout,"Event: ConfigureNotify\n");	
+	redraw();
 	break;
-      case Button2:
-	if (mmouse) {
+
+      case MotionNotify:
+	if (hasTooltipSupport()) {
+	  if (!hasTooltip()) {
+	    nTooltipTimer= currentTimeMillis();
+	    nTooltipX= report.xbutton.x;
+	    nTooltipY= report.xbutton.y;
+	    nTooltipButton= whichButton(report.xbutton.x, report.xbutton.y);
+	  } else {
+	    int nButton = whichButton(report.xbutton.x, report.xbutton.y);
+	    if (nButton != nTooltipButton) {
+	      hideTooltip();
+	      nTooltipTimer= -1;
+	      nTooltipX = report.xbutton.x;
+	      nTooltipY = report.xbutton.y;
+	      nTooltipButton = nButton;
+	      showTooltip(nTooltipButton, nTooltipX, nTooltipY);
+	    }
+	  }
+	}
+	break;
+      
+      case LeaveNotify:
+	if ( Config.Verbose ) fprintf(stdout,"Event: LeaveNotify\n");	
+	if (hasTooltip()) {
+	  hideTooltip();
+	  nTooltipHideTimer= currentTimeMillis();
+	}
+	nTooltipTimer= -1;
+	break;
+
+      case ButtonPress:	/* draw button pressed, don't launch *charkins*/
+	if (hasTooltip()) {
+	  hideTooltip();
+	  nTooltipHideTimer= currentTimeMillis();
+	}	switch (report.xbutton.button) {
+	case Button1:
 	  N = whichButton(report.xbutton.x, report.xbutton.y );
-	  if ( (N >= 0) && (N <= numb_of_apps) ) {
-	    button_pressed = N + MMASK;
+	  if ( (N >= 0) && (N <= NUMB_OF_APPS) ) {
+	    button_pressed = N + LMASK;
 	    redraw();
 	  }
-	  if ( Verbose ) 
+
+	  if ( Config.Verbose ) 
 	    fprintf(stdout,"Button 1:x=%d y=%d N=%d\n", 
-		    report.xbutton.x, report.xbutton.y, N+MMASK);
-	}
-	break;
-      case Button3:
-	N = whichButton(report.xbutton.x, report.xbutton.y );
-	if ( (N >= 0) && (N <= numb_of_apps) ) {
-	  button_pressed = N + RMASK;
-	  redraw();
-	}
-	if ( Verbose ) 
-	  fprintf(stdout,"Button 3:x=%d y=%d N=%d\n", 
-		  report.xbutton.x, report.xbutton.y, N+RMASK);
-	break;
-      }
-      break;
-    case ButtonRelease:	/* launch app here if still over button *charkins*/
-      switch (report.xbutton.button) {
-      case Button1:
-	N = whichButton(report.xbutton.x, report.xbutton.y );
-	if ( (N >= 0) && (N <= numb_of_apps) && (N == button_pressed))
-	  RunAppN( N + LMASK);
-	button_pressed=-1;
-	redraw();
-	if ( Verbose ) 
-	  fprintf(stdout,"Button 1:x=%d y=%d N=%d\n", 
-		  report.xbutton.x, report.xbutton.y, N+LMASK);	
-	break;
-      case Button2:
-	if (mmouse) {
+		    report.xbutton.x, report.xbutton.y, N+LMASK);	
+
+	  break;
+	case Button2:
+	  if (Config.mmouse) {
+	    N = whichButton(report.xbutton.x, report.xbutton.y );
+	    if ( (N >= 0) && (N <= NUMB_OF_APPS) ) {
+	      button_pressed = N + MMASK;
+	      redraw();
+	    }
+
+	    if ( Config.Verbose ) 
+	      fprintf(stdout,"Button 2:x=%d y=%d N=%d\n", 
+		      report.xbutton.x, report.xbutton.y, N+MMASK);
+
+	  }
+	  break;
+	case Button3:
 	  N = whichButton(report.xbutton.x, report.xbutton.y );
-	  if ( (N >= 0) && (N <= numb_of_apps) && (N == button_pressed))
-	    RunAppN( N + MMASK);
+	  if ( (N >= 0) && (N <= NUMB_OF_APPS) ) {
+	    button_pressed = N + RMASK;
+	    redraw();
+	  }
+
+	  if ( Config.Verbose ) 
+	    fprintf(stdout,"Button 3:x=%d y=%d N=%d\n", 
+		    report.xbutton.x, report.xbutton.y, N+RMASK);
+
+	  break;
+	}
+	break;
+      case ButtonRelease:   /* launch app here if still over button *charkins*/
+	switch (report.xbutton.button) {
+	case Button1:
+	  N = whichButton(report.xbutton.x, report.xbutton.y );
+	  if ( (N >= 0) && (N <= NUMB_OF_APPS) && (N == button_pressed))
+	    RunAppN( N + LMASK);
 	  button_pressed=-1;
 	  redraw();
-	  if ( Verbose ) 
+
+	  if ( Config.Verbose ) 
 	    fprintf(stdout,"Button 1:x=%d y=%d N=%d\n", 
-		    report.xbutton.x, report.xbutton.y, N+MMASK);
+		    report.xbutton.x, report.xbutton.y, N+LMASK);	
+
+	  break;
+	case Button2:
+	  if (Config.mmouse) {
+	    N = whichButton(report.xbutton.x, report.xbutton.y );
+	    if ( (N >= 0) && (N <= NUMB_OF_APPS) && (N == button_pressed))
+	      RunAppN( N + MMASK);
+	    button_pressed=-1;
+	    redraw();
+
+	    if ( Config.Verbose ) 
+	      fprintf(stdout,"Button 2:x=%d y=%d N=%d\n", 
+		      report.xbutton.x, report.xbutton.y, N+MMASK);
+
+	  }
+	  break;
+	case Button3:
+	  N = whichButton(report.xbutton.x, report.xbutton.y );
+	  if ( (N >= 0) && (N <= NUMB_OF_APPS) && (N == button_pressed))
+	    RunAppN( N + RMASK);
+	  button_pressed=-1;
+	  redraw();
+
+	  if ( Config.Verbose ) 
+	    fprintf(stdout,"Button 3:x=%d y=%d N=%d\n", 
+		    report.xbutton.x, report.xbutton.y, N+RMASK);
+
+	  break;
 	}
 	break;
-      case Button3:
-	N = whichButton(report.xbutton.x, report.xbutton.y );
-	if ( (N >= 0) && (N <= numb_of_apps) && (N == button_pressed))
-	  RunAppN( N + RMASK);
-	button_pressed=-1;
-	redraw();
-	if ( Verbose ) 
-	  fprintf(stdout,"Button 3:x=%d y=%d N=%d\n", 
-		  report.xbutton.x, report.xbutton.y, N+RMASK);
+      case DestroyNotify:
+
+	if ( Config.Verbose ) fprintf(stdout, "Bye\n");
+
+	destroyTooltip();
+	XFreeGC(display, gc);
+	XDestroyWindow(display,win);
+	XDestroyWindow(display,iconwin);
+	XCloseDisplay(display);
+	exit(0);
 	break;
       }
-      break;
-    case DestroyNotify:
-      if ( Verbose ) fprintf(stdout, "Bye\n");
-      XFreeGC(display, gc);
-      XDestroyWindow(display,win);
-      XDestroyWindow(display,iconwin);
-      XCloseDisplay(display);
-      if ( Verbose ) fprintf(stdout, "Bye\n");
-      exit(0);
-      break;
     }
+
+    usleep(50000);
+    nNow = currentTimeMillis();
+    if ( nTooltipTimer != -1 && 
+	 ( (nNow > nTooltipTimer + nTooltipShowDelay) || (nNow < nTooltipHideTimer + nTooltipReshowDelay) ) ) {
+      showTooltip(nTooltipButton, nTooltipX, nTooltipY);
+      nTooltipTimer = -1;
+    }
+    
   }
+
   return (0);
-}
+}/***********************************************************************/
 
 /***********************************************************************
  *   redraw
@@ -394,6 +404,7 @@ void redraw() {
   int offset;
   int bsize = 18;
 
+  if ( Config.Verbose ) fprintf(stdout,"In Redraw()\n");	
   space = 0;
   offset = 5;
   XCopyArea(display, template.pixmap, visible.pixmap, gc, 0, 0,
@@ -412,7 +423,7 @@ void redraw() {
       button_region[n].j = dest_y + bsize - 1; 
       
       /* Copy button images for valid apps */
-      if (  (n + 1) <= numb_of_apps ) {
+      if (  (n + 1) <= NUMB_OF_APPS ) {
         XCopyArea(display, buttons.pixmap, visible.pixmap, gc, 
 	          i * bsize, j * bsize, bsize, bsize, dest_x, dest_y);
       } 
@@ -445,8 +456,8 @@ void redraw() {
   flush_expose( iconwin ); 
   XCopyArea(display, visible.pixmap, iconwin, gc, 0, 0,
 	    visible.attributes.width, visible.attributes.height, 0, 0 );
-  if ( Verbose ) fprintf(stdout,"In Redraw()\n");	
-}
+
+}/***********************************************************************/
 
 /***********************************************************************
  *  whichButton
@@ -457,7 +468,7 @@ void redraw() {
 int whichButton( int x, int y ) {
   int index;
 
-  for ( index=0; index < numb_of_apps; index++ ) {
+  for ( index=0; index < NUMB_OF_APPS; index++ ) {
     if ( x >= button_region[index].x &&
 	 x <= button_region[index].i &&
 	 y >= button_region[index].y &&
@@ -466,60 +477,8 @@ int whichButton( int x, int y ) {
     }
   }
   return(-1);
-}
+}/***********************************************************************/
 
-/***********************************************************************
- * RunAppN(int app)
- *
- * Run the command given in the configuration file 'configfile'
- ***********************************************************************/
-void RunAppN( int app ) {
-  char *cmndstr;
-
-  cmndstr = Parse(app);                  // Get command to pass to system
-  if (Verbose) fprintf(stderr, "Command String: %s", cmndstr);
-  if (cmndstr != NULL) system(cmndstr);  // if there's a command, run it
-}
-
-/***********************************************************************
- * Parse(int app)
- *
- * Parses the file 'configfile' for command to execute.
- ***********************************************************************/
-
-char *Parse(int app) {
-  FILE *fp;
-  char *ptr  = NULL;
-  char *cmnd = NULL;
-  char *line = NULL;
-
-  if ((fp = fopen(configfile, "r")) == NULL) {  // Can't find config file
-    fprintf(stderr, "%s: Configuration File not found\n", configfile);
-    exit(-1);
-  }
-
-  do {                               // Read Lines in config file
-    line = readln(fp);
-    if (line == NULL) break;         // if end of file, quit
-    if ( (line[0] != '#') && (line[0] != '\n'))  // Ignore comments and blanks 
-      if ( atoi(line) == app) break; // if we've found our app, quit looking
-  } while (1);
-
-  if (line == NULL) return(NULL);    // if we didn't find in, that just ignore
-
-  ptr = strchr(line,'\t');                      // find first tab
-  if (ptr == NULL)   ptr = strchr(line,' ');    // or space charater
-  if (ptr == NULL)   return(NULL);              // if neither are there, its a badly
-                                                // formatted line.  Ignore
-  ptr++;
-  cmnd = malloc((strlen(ptr)+1)*sizeof(char));  
-
-  strcpy(cmnd, ptr);                 // Everything after number is passed to system
-
-  free(line);
-  fclose(fp);
-  return(cmnd);
-}
 
 /***********************************************************************
  *   getPixmaps
@@ -533,61 +492,66 @@ char *Parse(int app) {
  *          later copied onto Pixmap visible.
  ***********************************************************************/
 void getPixmaps() {
+  int loaded = 0;
   template.attributes.valuemask |= (XpmReturnPixels | XpmReturnExtensions);
   visible.attributes.valuemask  |= (XpmReturnPixels | XpmReturnExtensions);
   buttons.attributes.valuemask  |= (XpmReturnPixels | XpmReturnExtensions);
   
+
+  if (Config.Verbose) fprintf(stdout, "In getPixmaps\n");
   /* Template Pixmap. Never Drawn To. */
   if ( XpmCreatePixmapFromData(	display, rootwin, backdrop_xpm,
 				&template.pixmap, &template.mask, 
-				&template.attributes) != XpmSuccess ) {
-    fprintf(stderr, "Can't Create 'template' Pixmap\n");
-    exit(1);
-  }
+				&template.attributes) != XpmSuccess )
+    err_mess(FAILTMPL, NULL);
+
   /* Visible Pixmap. Copied from template Pixmap and then drawn to. */
   if ( XpmCreatePixmapFromData(	display, rootwin, backdrop_xpm,
 				&visible.pixmap, &visible.mask, 
-				&visible.attributes) != XpmSuccess ) {
-    fprintf(stderr, "Can't Create 'visible' Pixmap");
-    exit(1);
-  }
+				&visible.attributes) != XpmSuccess )
+    err_mess(FAILVIS, NULL);
   
   /* Button Pixmap.  */
-  if ( XpmCreatePixmapFromData(	display, rootwin, buttons_xpm,
-				&buttons.pixmap, &buttons.mask, 
-				&buttons.attributes) != XpmSuccess ) {
-    fprintf(stderr, "Can't Create 'buttons' Pixmap");
-    exit(1);
+  if ( access( Config.buttonfile, R_OK  ) == 0 ) {
+    /* load buttons from file */
+    if ( XpmReadFileToPixmap( display, rootwin, Config.buttonfile,
+      			      &buttons.pixmap, &buttons.mask, 
+			      &buttons.attributes) != XpmSuccess ) {
+      err_mess(FAILBUT, NULL); 
+    } else {
+      loaded = 1;
+    }
   }
+  if (! loaded) { 
+    /* Use Builtin Button Pixmap.  */
+    if (Config.Verbose) fprintf(stdout, "Using builtin buttons pixmap\n");
+    if ( XpmCreatePixmapFromData( display, rootwin, buttons_xpm,
+				  &buttons.pixmap, &buttons.mask, 
+				  &buttons.attributes) != XpmSuccess )
+      err_mess(FAILBUT, NULL); 
+  }
+  if (Config.Verbose) fprintf(stdout, "Leaving getPixmaps\n");
+
+}/*********************************************************************/
+
+void SetWmHints() {
+  XWMHints *xwmhints;
+
+  xwmhints = XAllocWMHints();
+  xwmhints->flags = WindowGroupHint | IconWindowHint | StateHint;	
+  xwmhints->icon_window = iconwin;
+  xwmhints->window_group = win;
+  xwmhints->initial_state = WithdrawnState;  
+  XSetWMHints( display, win, xwmhints );
+	XFree(xwmhints);
+	xwmhints = NULL;
 }
 
-/***********************************************************************
- *   flush_expose
- *
- *   Everyone else has one of these... Can't hurt to throw it in.
- ***********************************************************************/
-int flush_expose(Window w) {
-  XEvent      dummy;
-  int         i=0;
-  
-  while (XCheckTypedWindowEvent(display, w, Expose, &dummy)) i++;
-  return(i);
+void SetClassHints() {
+  XClassHint xclasshint;
+
+  xclasshint.res_name = "wmbutton";
+  xclasshint.res_class = "Wmbutton";
+  XSetClassHint( display, win, &xclasshint );
 }
 
-/***********************************************************************
- * 		show_usage
- ***********************************************************************/
-void show_usage() {
-  fprintf(stderr,"\n");
-  fprintf(stderr,"usage: %s [-g geometry] [-d dpy] [-v] [-f <configfile>]\n",app_name);
-  fprintf(stderr,"\n");
-  fprintf(stderr," wmbutton Ver %d Rel %d\n",VER,REL);
-  fprintf(stderr,"\n");
-  fprintf(stderr,"-g  <geometry>   Window Geometry - ie: 64x64+10+10\n");
-  fprintf(stderr,"-d  <display>    Display -  ie: 127.0.0.1:0.0\n"); 
-  fprintf(stderr,"-f  <filename>   Full path to configuration file.\n");
-  fprintf(stderr,"-v               Verbose Mode. \n");
-  fprintf(stderr,"-h               Help. This message.\n");
-  fprintf(stderr,"-m               Enable Middle Mouse functionality.\n\n");
-  exit(-1);
-}
